/************************************************************************
 *   IRC - Internet Relay Chat, src/m_setname.c
 *
 *   Written for UltimateIRCd by ShadowRealm Creations.
 *   Copyright (C) 1997-2000 Infomedia Inc.
 *
 *   Adapted to ircd-hybrid by Robert Jameson (rj)
 *
 *
 * 
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include "common.h"
#include "assert.h"
#include "m_commands.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "channel.h"
#include "irc_string.h"
#include "s_debug.h"

extern int valid_hostname(const char* hostname);
extern int valid_username(const char* username);

/*
 * m_functions execute protocol messages on this server:
 *
 *      cptr    is always NON-NULL, pointing to a *LOCAL* client
 *              structure (with an open socket connected!). This
 *              identifies the physical socket where the message
 *              originated (or which caused the m_function to be
 *              executed--some m_functions may call others...).
 *
 *      sptr    is the source of the message, defined by the
 *              prefix part of the message if present. If not
 *              or prefix not found, then sptr==cptr.
 *
 *              (!IsServer(cptr)) => (cptr == sptr), because
 *              prefixes are taken *only* from servers...
 *
 *              (IsServer(cptr))
 *                      (sptr == cptr) => the message didn't
 *                      have the prefix.
 *
 *                      (sptr != cptr && IsServer(sptr) means
 *                      the prefix specified servername. (?)
 *
 *                      (sptr != cptr && !IsServer(sptr) means
 *                      that message originated from a remote
 *                      user (not local).
 *
 *              combining
 *
 *              (!IsServer(sptr)) means that, sptr can safely
 *              taken as defining the target structure of the
 *              message in this server.
 *
 *      *Always* true (if 'parse' and others are working correct):
 *
 *      1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *      2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *              *cannot* be a local connection, unless it's
 *              actually cptr!). [MyConnect(x) should probably
 *              be defined as (x == x->from) --msa ]
 *
 *      parc    number of variable parameter strings (if zero,
 *              parv is allowed to be NULL)
 *
 *      parv    a NULL terminated list of parameter pointers,
 *
 *                      parv[0], sender (prefix string), if not present
 *                              this points to an empty string.
 *                      parv[1]...parv[parc-1]
 *                              pointers to additional parameters
 *                      parv[parc] == NULL, *always*
 *
 *              note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                      non-NULL pointers.
 */

/*
** m_newhost
**      parv[0] = sender prefix
**      parv[1] = target (optional)
**      parv[2] = hostname
*/
int     m_newhost(struct Client *cptr,
                  struct Client *sptr,
                  int parc,
                  char *parv[])
{
  struct Client *acptr;
  char  *hostname, *target;

  target = parv[1];
  hostname = parv[2];

  if (!IsPrivileged(cptr))
    {
      sendto_one(cptr, form_str(ERR_NOPRIVILEGES),
                 me.name, parv[0], "NEWHOST");
      return 0;
    }

  if (parc < 2 || *parv[1] == '\0')
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "NEWHOST");
      return 0;
    }

  if (parc == 2)
    {
      hostname = parv[1];
      target = sptr->name;
    }

  if (!valid_hostname(hostname) || strlen(hostname) > HOSTLEN) {
    sendto_one(sptr, ":%s NOTICE %s :*** Error: newhost [%s] is invalid",
               me.name, parv[0], hostname);
    return 0;
  }

  if ((acptr = find_person(target, NULL))) {
    sendto_ops_flags(FLAGS_SPY,
                     "%s is setting %s\'s (%s@%s) host to \2%s\2",
                     sptr->name, acptr->name,
                     acptr->username, acptr->host, hostname);
      if (sptr == acptr) {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Host: \2%s\2",
                    me.name, sptr->name, hostname);
      } else {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Host: %s set your host to \2%s\2",
                    me.name, acptr->name, sptr->name, hostname);
         if (MyClient(sptr))
         sendto_one(sptr, ":%s NOTICE %s :** New Host: \2%s\2\'s host has been set to \2%s\2",
                    me.name, sptr->name, acptr->name, hostname);
      }
    sendto_serv_butone(cptr, ":%s NEWHOST %s :%s",
                       sptr->name,
                       target,
                       hostname);
    ircsprintf(acptr->host, "%s", hostname);
  }
  else {
    sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, sptr->name, target);
    return 0;
  }

#ifdef  DEBUGMODE
  Debug((DEBUG_NOTICE, "NEWHOST: %s %s", target, hostname));
#endif
  return 0;
}


/*
** m_newident
**      parv[0] = sender prefix
**      parv[1] = target (optional)
**      parv[2] = username
*/
int     m_newident(struct Client *cptr,
                  struct Client *sptr,
                  int parc,
                  char *parv[])
{
  struct Client *acptr;
  char  *username, *target;

  target = parv[1];
  username = parv[2];

  if (!IsPrivileged(cptr))
    {
      sendto_one(cptr, form_str(ERR_NOPRIVILEGES),
                 me.name, parv[0], "NEWIDENT");
      return 0;
    }

  if (parc < 2 || *parv[1] == '\0')
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "NEWIDENT");
      return 0;
    }

  if (parc == 2)
    {
      username = parv[1];
      target = sptr->name;
    }

  if (!valid_username(username) || strlen(username) > USERLEN) {
    sendto_one(sptr, ":%s NOTICE %s :*** Error: newident [%s] is invalid",
               me.name, parv[0], username);
    return 0;
  }

  if ((acptr = find_person(target, NULL))) {
    sendto_ops_flags(FLAGS_SPY,
                     "%s is setting %s\'s (%s@%s) ident to \2%s\2",
                     sptr->name, acptr->name,
                     acptr->username, acptr->host, username);
      if (sptr == acptr) {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Ident: \2%s\2",
                    me.name, sptr->name, username);
      } else {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Ident: %s set your ident to \2%s\2",
                    me.name, acptr->name, sptr->name, username);
         if (MyClient(sptr))
         sendto_one(sptr, ":%s NOTICE %s :** New Ident: \2%s\2\'s ident has been set to \2%s\2",
                    me.name, sptr->name, acptr->name, username);
      }
    sendto_serv_butone(cptr, ":%s NEWIDENT %s :%s",
                       sptr->name,
                       target,
                       username);
    ircsprintf(acptr->username, "%s", username);
  }
  else {
    sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, sptr->name, target);
    return 0;
  }

#ifdef  DEBUGMODE
  Debug((DEBUG_NOTICE, "NEWIDENT: %s %s", target, username));
#endif
  return 0;
}


/*
** m_newname
**      parv[0] = sender prefix
**      parv[1] = target (optional)
**      parv[2] = realname
*/
int     m_newname(struct Client *cptr,
                  struct Client *sptr,
                  int parc,
                  char *parv[])
{
  struct Client *acptr;
  char  *realname, *target;

  target = parv[1];
  realname = parv[2];

  if (parc < 3)
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "NEWNAME");
      return 0;
    }

  if (!IsPrivileged(cptr) && (find_person(target, NULL) != sptr))
    {
      sendto_one(sptr, form_str(ERR_USERSDONTMATCH), me.name, parv[0]);
      return 0;
    }

  if (strlen(realname) > REALLEN) {
    sendto_one(sptr, ":%s NOTICE %s :*** Error: newname [%s] is invalid",
               me.name, parv[0], realname);
    return 0;
  }

  if ((acptr = find_server(target))) {
    sendto_ops_flags(FLAGS_SERVNOTICE,
                     "%s is setting %s\'s (%s@%s) name to \2%s\2",
                     sptr->name, acptr->name, realname);
    sendto_serv_butone(cptr, ":%s NEWNAME %s :%s",
                       sptr->name,
                       target,
                       realname);
    ircsprintf(acptr->info, "%s", realname);
  } else if ((acptr = find_person(target, NULL))) {
    sendto_ops_flags(FLAGS_SPY,
                     "%s is setting %s\'s (%s@%s) name to \2%s\2",
                     sptr->name, acptr->name,
                     acptr->username, acptr->host, realname);
      if (sptr == acptr) {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Name: \2%s\2",
                    me.name, sptr->name, realname);
      } else {
         if (MyClient(acptr))
         sendto_one(acptr, ":%s NOTICE %s :** New Name: %s set your name to \2%s\2",
                    me.name, acptr->name, sptr->name, realname);
         if (MyClient(sptr))
         sendto_one(sptr, ":%s NOTICE %s :** New Name: \2%s\2\'s name has been set to \2%s\2",
                    me.name, sptr->name, acptr->name, realname);
      }
    sendto_serv_butone(cptr, ":%s NEWNAME %s :%s",
                       sptr->name,
                       target,
                       realname);
    ircsprintf(acptr->info, "%s", realname);
  }
  else {
    sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, sptr->name, target);
    return 0;
  }

#ifdef  DEBUGMODE
  Debug((DEBUG_NOTICE, "NEWNAME: %s %s", target, realname));
#endif
  return 0;
}

