#include "client.h"
#include "common.h"
#include "s_log.h"
#include "irc_string.h"
#include "s_bsd.h"
#include "ircd.h"
#include "send.h"
#include "md5.h"
#include "s_conf.h"
#include "s_serv.h"
#include <string.h>
#include <stdlib.h>
#include <time.h>

#ifdef CHALLENGERESPONSE
#ifdef CHALLENGERESPONSE_FALLBACK
extern time_t cr_challengeexpirytime;
void cr_checkfallback() {
  /* The cr_challengeexpirytime was reached. So, let's look through
     or list for servers we are connecting to which we haven't got
     a password for (aka no RESP received), and the connect happened 
     more than CHALLENGERESPONSE_EXPIRY seconds ago. If we find one,
     we revert to PASS. By lagging servers it'd be possible to provoke
     a fallback to PASS, so this could be exploited.
     - einride.
  */  
  time_t nextexpiry = 0;
  struct Client * cptr;
  struct ConfItem *c_conf, *n_conf;
  for (cptr=GlobalClientList; cptr; cptr = cptr->next_local_client) {
    if (Challenged(cptr) && !Responded(cptr)) {
      if (cptr->firsttime < (CurrentTime - CHALLENGERESPONSE_EXPIRY)) {
	/* Revert to PASS */
	log(L_NOTICE, "CR expiry time reached for %s - fallback", cptr->name ? cptr->name : "<unknown>");
	c_conf = find_conf_by_name(cptr->name, CONF_CONNECT_SERVER);
	n_conf = find_conf_by_name(cptr->name, CONF_NOCONNECT_SERVER);
	if (c_conf && n_conf) {
	  sendto_ops("Trying PASS to authenticate with %s", cptr->name);
	  if (!EmptyString(c_conf->passwd))
	    sendto_one(cptr, "PASS %s :TS", c_conf->passwd);
	  send_capabilities(cptr, (c_conf->flags & CONF_FLAGS_ZIP_LINK));
	  sendto_one(cptr, "SERVER %s 1 :%s",
		     my_name_for_link(me.name, n_conf), me.info);
	} else {
	  sendto_ops("Can't find C/N lines on CR expiry - dropping %s", cptr->name);
	  log(L_NOTICE, "Can't find C/N lines on CR expiry - dropping %s", cptr->name);
	  exit_client(cptr, cptr, cptr, "No C/N lines");
	}
      } else {
	if (!nextexpiry || nextexpiry > (cptr->firsttime + CHALLENGERESPONSE_EXPIRY))
	  nextexpiry = (cptr->firsttime + CHALLENGERESPONSE_EXPIRY);
      }
    }
  }
  cr_challengeexpirytime=nextexpiry;
}
#endif

void cr_sendchallenge(struct Client * cptr, struct ConfItem * n_conf) {
  /* Create a challenge using random data and sending it as a hex value.
     Store this hex value in cptr->passwd, don't want more fields in cptr.
     As this is sent before anything else, we pass the server name too so
     the receiving end can look up our C/N lines.
  */
  extern char* get_salt(char*);
  char chall[17]="";
  unsigned int c;
  int i;
  srandom(time(0));
  for (i=0; i<16; i++) {
    c = (unsigned int) (16.0 * (random() / (RAND_MAX + 1.0)));
    chall[i] = (c>=10) ? (c - 10 + 'A' ) : (c + '0');
  }
  chall[16]=0;
  sendto_one(cptr, "CHALL %s %s %s :TS", me.name, chall, get_salt(n_conf->passwd));
  strncpy_irc(cptr->name, n_conf->name, HOSTLEN);
  strcpy(cptr->passwd, chall);
  SetChallenged(cptr);
  sendto_ops("Sent password challenge to %s", cptr->name);
#ifdef CHALLENGERESPONSE_FALLBACK
  if (!cr_challengeexpirytime)
    cr_challengeexpirytime = CurrentTime + CHALLENGERESPONSE_EXPIRY;
#endif
}

void cr_hashstring(char * src, char * trg) {
  unsigned char md[16];
  MD5_CTX ctx;
  MD5Init(&ctx);
  MD5Update(&ctx, src, strlen(src));
  MD5Final(md, &ctx);
  ircsprintf(trg, "%08x", * (unsigned int *) &md[0]);
  ircsprintf(trg + 8, "%08x", * (unsigned int *) &md[4]);
  ircsprintf(trg + 16, "%08x", * (unsigned int *) &md[8]);
  ircsprintf(trg + 24, "%08x", * (unsigned int *) &md[12]);
}

void cr_sendresponse(struct Client * cptr, struct ConfItem * c_conf, char * chall, char* salt) {
  extern char * libshadow_md5_crypt(const char *pw, const char *salt);
  char work[PASSWDLEN*2], hash[36];
  strncpy_irc(work, chall, PASSWDLEN);
  strncpy_irc(work + strlen(work), c_conf->passwd ? libshadow_md5_crypt(c_conf->passwd, salt) : "", PASSWDLEN);
  cr_hashstring(work, hash);
  sendto_one(cptr, "RESP %s", hash);
  SetResponded(cptr);
  sendto_ops("Sent password response to %s", cptr->name);
}

void cr_gotresponse(struct Client * cptr, char * resp) {
  struct ConfItem * n_conf;
  char work[PASSWDLEN*2], hash[36];
  n_conf = find_conf_by_name(cptr->name, CONF_NOCONNECT_SERVER);
  if (n_conf && cptr->passwd[0]) {
    strncpy_irc(work, cptr->passwd, PASSWDLEN);
    strncpy_irc(work + strlen(work), n_conf->passwd ? n_conf->passwd : "", PASSWDLEN);
    cr_hashstring(work, hash);
    if (!strcmp(resp, hash)) {
      strncpy_irc(cptr->passwd, n_conf->passwd, PASSWDLEN);
      sendto_ops("Got a good password response from %s", cptr->name);
    } else {
      sendto_ops("Failed password response from %s", cptr->name);
    }
  }
}

#endif

